"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerStreamingCall = void 0;
/**
 * A server streaming RPC call. The client provides exactly one input message
 * but the server may respond with 0, 1, or more messages.
 */
class ServerStreamingCall {
    constructor(serviceInfo, methodInfo, requestHeaders, requestMessage, responseHeaders, responseStream, responseStatus, responseTrailers, cancelFn) {
        this.serviceInfo = serviceInfo;
        this.methodInfo = methodInfo;
        this.requestType = methodInfo.I;
        this.responseType = methodInfo.O;
        this.requestHeaders = requestHeaders;
        this.requestMessage = requestMessage;
        this.responseHeaders = responseHeaders;
        this.responseStream = responseStream;
        this.responseStatus = responseStatus;
        this.responseTrailers = responseTrailers;
        this.cancelFn = cancelFn;
    }
    /**
     * Cancel this call.
     */
    cancel() {
        this.cancelFn();
    }
    /**
     * Instead of awaiting the response status and trailers, you can
     * just as well await this call itself to receive the final outcome.
     * You should first setup some listeners to the `requestMessage` to
     * see the actual messages the server replied with.
     */
    then(onfulfilled, onrejected) {
        return this.promiseFinished().then(value => onfulfilled ? Promise.resolve(onfulfilled(value)) : value, reason => onrejected ? Promise.resolve(onrejected(reason)) : Promise.reject(reason));
    }
    promiseFinished() {
        return __awaiter(this, void 0, void 0, function* () {
            let [responseHeaders, responseStatus, responseTrailers] = yield Promise.all([this.responseHeaders, this.responseStatus, this.responseTrailers]);
            return {
                serviceInfo: this.serviceInfo,
                methodInfo: this.methodInfo,
                requestType: this.requestType,
                responseType: this.responseType,
                requestHeaders: this.requestHeaders,
                requestMessage: this.requestMessage,
                responseHeaders,
                responseStatus,
                responseTrailers,
            };
        });
    }
}
exports.ServerStreamingCall = ServerStreamingCall;
//# sourceMappingURL=server-streaming-call.js.map